using System.ComponentModel;
using Dapfor.Net.Data;
using Dapfor.Net.Formats;

namespace Dapfor.Net.Grid.BasketViewer.BusinessLevel
{
    internal class Industry : INotifyPropertyChanged
    {
        private readonly string _industryName;
        private readonly InstrumentClass _instrumentClass;
        private readonly ThreadSafeBindingList<Share> _shares = new ThreadSafeBindingList<Share>();
        private decimal? _marketValue;

        public Industry(string industryName, InstrumentClass instrumentClass)
        {
            _industryName = industryName;
            _instrumentClass = instrumentClass;
        }

        public string IndustryName
        {
            get { return _industryName; }
        }

        public InstrumentClass InstrumentClass
        {
            get { return _instrumentClass; }
        }

        [Format("### ### ###.00")]
        public decimal MarketValue
        {
            get
            {
                //Cache purposes: compute industry's market value
                if (_marketValue == null)
                {
                    _marketValue = 0;
                    foreach (Share share in _shares)
                    {
                        _marketValue += share.MarketValue;
                    }
                }
                return _marketValue.Value;
            }
        }

        //Use 2 decimals for industries 
        [Format("### ### ###.00", null, " ")]
        public decimal MarketValueEur
        {
            get
            {
                //Do we need convert market value to EUR?
                return MarketValue;
            }
        }

        public decimal MarketPart
        {
            get
            {
                if (_instrumentClass.MarketPart != 0)
                {
                    return _instrumentClass.MarketPart*MarketValue/_instrumentClass.MarketValue;
                }
                return 0;
            }
        }

        [Format("#.00", null, " %")]
        public decimal MarketPartPercent
        {
            get { return 100*MarketPart; }
        }

        //Declarative databinding indicates grid to build hierarchy
        [HierarchicalField]
        public ThreadSafeBindingList<Share> Shares
        {
            get { return _shares; }
        }

        #region INotifyPropertyChanged implementation

        public event PropertyChangedEventHandler PropertyChanged;

        private void FirePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion

        public void AddShare(Share share)
        {
            if (share != null)
            {
                share.PropertyChanged += OnSharePropertyChanged;
                _shares.Add(share);
                _marketValue = null;
            }
        }


        private void OnSharePropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "Price":
                    _marketValue = null;
                    FirePropertyChanged("MarketValue");
                    FirePropertyChanged("MarketValueEur");
                    break;
            }
        }
    }
}